--15 slot/8 bank quickslot bar, lockbutton disables buttons and slot changing
--operations, but can still drag bar and copy slots from bar to elsewhere.
--Based on the original work of PagedSlots by D.H1cks and incorporating modifications used in
--Big Paged Slots by Amethadan.

import "Turbine.Gameplay";
import "Turbine.UI.Extensions";
import "Turbine.Debug"

pagedSlotsWindow = class( Turbine.UI.Extensions.SimpleWindow );

function pagedSlotsWindow:Constructor()
	Turbine.UI.Extensions.SimpleWindow.Constructor( self );

	-- load the last settings saved
	self:LoadSettings();

	--[[ Here are the global variables ]]--
	self.lockbar = true;
	self.pageNumber = 1;
	self.draggingShortcut = false;
	self.draggedOut = false;
	self.swappingShortcuts = false;
	self.ReplacedShortcut = nil;
	self.loading = false;
	self.bgColor = Turbine.UI.Color(0.3,0.5,0.5,0.5)

	--[[ set window properties ]]--
	self:SetSize( 580, 35 );
	self:SetBackColor(Turbine.UI.Color(.2,0,0,0));
	self:SetPosition(self.settings.positionX,self.settings.positionY);
	self:SetOpacity( 1 );
	self:SetZOrder(1);
	self:SetAllowDrop( true );
	self:SetVisible(true);
	
	--[[ Add the label to the window ]]--
	self.pageLabel = Turbine.UI.Label();
	self.pageLabel:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);

	self.pageLabel:SetBackColor(Turbine.UI.Color(0,0,0,0));
	self.pageLabel:SetForeColor(Turbine.UI.Color(1,1,1,1));
	self.pageLabel:SetBackColorBlendMode(Turbine.UI.BlendMode.Undefined);
	self.pageLabel:SetZOrder(99);
	self.pageLabel:SetParent(self);

	--[[ Add the page up and down buttons ]]--
	self.pageUpButton = Turbine.UI.Button();
	self.pageDownButton = Turbine.UI.Button();

	self.pageUpButton:SetParent(self);
	self.pageUpButton:SetBackColor(Turbine.UI.Color(0,0,0,0));
	self.pageUpButton:SetZOrder(50);
	self.pageUpButton:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	self.pageUpButton:SetBackColorBlendMode(Turbine.UI.BlendMode.Undefined);


	self.pageDownButton:SetParent(self);
	self.pageDownButton:SetBackColor(Turbine.UI.Color(1,1,1,1));
	self.pageDownButton:SetZOrder(50);
	self.pageDownButton:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	self.pageDownButton:SetBackColorBlendMode(Turbine.UI.BlendMode.Undefined);

	--[[ Add the toggle direction button ]]--
	self.toggleButton = Turbine.UI.Button();
	self.toggleButton:SetParent(self);
	self.toggleButton:SetBackColor(Turbine.UI.Color(0,0,0,0));
	self.toggleButton:SetZOrder(50);
	self.toggleButton:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	self.toggleButton:SetBackColorBlendMode(Turbine.UI.BlendMode.Undefined);
	
		--[[ Add the toggle lock button ]]--
	self.lockButton = Turbine.UI.Button();
	self.lockButton:SetParent(self);
	self.lockButton:SetBackColor(Turbine.UI.Color(0,0,0,0));
	self.lockButton:SetZOrder(50);
	self.lockButton:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	self.lockButton:SetBackColorBlendMode(Turbine.UI.BlendMode.Undefined);
			
		--[[ Add the bar number button ]]--
	self.label2Button = Turbine.UI.Button();
	self.label2Button:SetParent(self);
	self.label2Button:SetBackColor(Turbine.UI.Color(0,0,0,0));
	self.label2Button:SetZOrder(0);
	self.label2Button:SetBlendMode(Turbine.UI.BlendMode.AlphaBlend);
	self.label2Button:SetBackColorBlendMode(Turbine.UI.BlendMode.Undefined);

	--[[ set values for the quickslot item ]]--
	self.quickslotSize = 35;
	self.quickslots = { };

	--[[ set the location of the quickslot, then initialize 
           to the first item in the list  ]]--
	for i = 1,15,1 do
		self.quickslots[i] = Turbine.UI.Lotro.Quickslot();		
		self.quickslots[i]:SetParent( self );
		self.quickslots[i]:SetZOrder(100);
		self.quickslots[i]:SetSize(self.quickslotSize, self.quickslotSize);
	end
	self:SetControls();
	self:SetQuickslots();
	local travel = self;
	local mainWindow = self;

	-- add callback for event handling --
function AddCallback(object, event, callback)
    if (object[event] == nil) then
        object[event] = callback;
    else
        if (type(object[event]) == "table") then
            table.insert(object[event], callback);
        else
            object[event] = {object[event], callback};
        end
    end
    return callback;
end
function RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil;
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);
            for i = 1, size do
                if (object[event][i] == callback) then
                    table.remove(object[event], i);
                    break;
                end
            end
        end
    end
end


	-- Hook up the visibility to show only when in combat. 
    -- Enter "--" in front of the next four lines to have the bar always show 
    -- Can then be toggled on off using the "/ls2 t" command
--	self.player = Turbine.Gameplay.LocalPlayer.GetInstance();
--	AddCallback(self.player, "InCombatChanged", function(sender, args)
--	self:SetVisible( self.player:IsInCombat() );
--	end);

	------------------------------------------
	--[[ Below are all the event handlers ]]--
	------------------------------------------

	local isMoving = false;
	local x = 0;
	local y = 0;

	self.MouseDown = function(sender, args )
		if (args.Button == Turbine.UI.MouseButton.Left) then
			isMoving = true;
			x = args.X;
			y = args.Y;
		end
	end
	
----------------------------------------------
	
	self.MouseUp = function(sender, args )
		if (args.Button == Turbine.UI.MouseButton.Left) then
			isMoving = false;

			local one,two = self:GetPosition();
			self.settings.positionX = one;
			self.settings.positionY = two;
			self:SaveSettings();
		end
	end

----------------------------------------------

	self.MouseMove = function(sender, args )
		if (isMoving) then
			local oldX,oldY = self:GetPosition();
			self:SetPosition(oldX + args.X - x,oldY + args.Y - y);	
		end
	end

----------------------------------------------

	self.toggleButton.Click = function(sender, args)
	if (not self.lockbar) then
		if(self.settings.orientation == 1) then
			self.settings.orientation = 2;
		else
			self.settings.orientation = 1;
		end
	else
	end
		self:SetControls();
		self:SetQuickslots();
		self:SaveSettings();
	end

----------------------------------------------

	self.lockButton.Click = function(sender, args)
	  if(self.lockbar == true) then
			self.lockbar = false;
			Turbine.Shell.WriteLine("Bar 2 Unlocked");
		else
			self.lockbar = true;
			Turbine.Shell.WriteLine("Bar 2 Locked");						
	  end
		self:SetControls();
		self:SetQuickslots();
		self:SaveSettings();
	end
	
----------------------------------------------	
	
	self.pageUpButton.Click = function(sender, args)
 	if (not self.lockbar) then
		self.pageNumber = self.pageNumber + 1;

		-- stop it at page 8
		if(self.pageNumber > 8) then
			self.pageNumber = 8;
 	else
	end	
		end
  		self:SetControls();
		self:SetQuickslots();
	end

----------------------------------------------

	self.pageDownButton.Click = function(sender, args)
	if (not self.lockbar) then
		self.pageNumber = self.pageNumber - 1;

		-- stop it at page 1
		if(self.pageNumber < 1) then
			self.pageNumber = 1;
	else
	end
		end
		self:SetControls();
		self:SetQuickslots();
	end

----------------------------------------------

	self.MouseEnter = function(sender,args)
		self:SetOpacity( 1 )
	end

----------------------------------------------

	self.MouseLeave = function(sender,args)
		self:SetOpacity( 1 )
	end

----------------------------------------------

	-- put a message in chat when the plugin is loaded      
   Plugins["LockSlots2"].Load = function(sender,args)
      Turbine.Shell.WriteLine("LockSlots2 has been loaded, type /ls2 for options"); 
   end

end

--------------------------------------------------------------------------------------------

function pagedSlotsWindow:SetControls()
	self.pageLabel:SetText(string.format("%d",self.pageNumber));

--Horizontal layout settings
	if(self.settings.orientation == 1) then
		self:SetSize( 580, 35 );

		self.toggleButton:SetBackground("Delone/LockSlots2/Resources/toggle.tga");
		self.toggleButton:SetSize(17,17);
		self.toggleButton:SetPosition(0,0);
		self.toggleButton:SetVisible( true );

		self.label2Button:SetBackground("Delone/LockSlots2/Resources/label2.tga");
		self.label2Button:SetSize(8,12);
		self.label2Button:SetPosition(0,22);
		self.label2Button:SetVisible( true );
		
		self.lockButton:SetBackground("Delone/LockSlots2/Resources/lock.tga");
		self.lockButton:SetSize(17,17);
		self.lockButton:SetPosition(560,17);
		self.lockButton:SetVisible( true );

		self.pageLabel:SetSize(17,17)
		self.pageLabel:SetPosition(560,-1);
		self.pageLabel:SetVisible( true );

		self.pageUpButton:SetBackground("Delone/LockSlots2/Resources/UpArrow.tga");
		self.pageUpButton:SetSize(16,16);
		self.pageUpButton:SetPosition(540,0);
		self.pageUpButton:SetVisible( true );

		self.pageDownButton:SetBackground("Delone/LockSlots2/Resources/DownArrow.tga");
		self.pageDownButton:SetSize(16,16);
		self.pageDownButton:SetPosition(540,17);
		self.pageDownButton:SetVisible( true );

--Vertical layout settings
	else
		self:SetSize( 35, 580);

		self.toggleButton:SetBackground("Delone/LockSlots2/Resources/toggle.tga");
		self.toggleButton:SetSize(17,17);
		self.toggleButton:SetPosition(0,0);
		self.toggleButton:SetVisible( true );
				
		self.label2Button:SetBackground("Delone/LockSlots2/Resources/label2.tga");
		self.label2Button:SetSize(8,12);
		self.label2Button:SetPosition(22,0);
		self.label2Button:SetVisible( true );
	
		self.lockButton:SetBackground("Delone/LockSlots2/Resources/lock.tga");
		self.lockButton:SetSize(17,17);
		self.lockButton:SetPosition(0,560);
		self.lockButton:SetVisible( true );

		self.pageLabel:SetSize(17,17)
		self.pageLabel:SetPosition(17,560);
		self.pageLabel:SetVisible( true );

		self.pageUpButton:SetBackground("Delone/LockSlots2/Resources/RightArrow.tga");
		self.pageUpButton:SetSize(16,16);
		self.pageUpButton:SetPosition(17,540);
		self.pageUpButton:SetVisible( true );

		self.pageDownButton:SetBackground("Delone/LockSlots2/Resources/LeftArrow.tga");
		self.pageDownButton:SetSize(16,16);
		self.pageDownButton:SetPosition(0,540);
		self.pageDownButton:SetVisible( true );


	end
end

function pagedSlotsWindow:SetQuickslots()

	for i = 1, 15, 1 do
		-- set the properties of the quickslot used to hold the shortcut
	
		if(self.settings.orientation == 1) then
			self.quickslots[i]:SetPosition(12 + (i - 1) * self.quickslotSize,0);
		else
			self.quickslots[i]:SetPosition(0,10 + (i - 1) * self.quickslotSize);
		end

		local type = self.settings.shortcuts[i + (self.pageNumber - 1) * 15].Type;
		local data = self.settings.shortcuts[i + (self.pageNumber - 1) * 15].Data;
		
		
   
      myShortcut = Turbine.UI.Lotro.Shortcut(type,data);
 	if (not self.lockbar) then
		self.quickslots[i]:SetShortcut(myShortcut);
		self.quickslots[i]:SetAllowDrop(true);
 	else
		self.quickslots[i]:SetShortcut(myShortcut);
		self.quickslots[i]:SetAllowDrop(false); 	
	end 		
		self.quickslots[i]:SetZOrder(100);		


		--[[ register the event to handle adding a new shortcut ]]--
		self.quickslots[i].ShortcutChanged = function(sender,args)
		   -- update and save the settings
         self:ChangeShortcut(i);
            			
         -- check if we are going to perform a swap
 	if (not self.lockbar) then
			if(self.swappingShortcuts and not self.deleting) then
			   
			   -- clear the swap flag because this event will fire when we set
			   -- old slot, and we don't want to do a double swap
	         self.swappingShortcuts = false;
            
            -- set the old quickslot with the shortcut we are replacing
            if(self.draggedIndex ~= 0) then		
				  self.quickslots[self.draggedIndex]:SetShortcut(self.ReplacedShortcut);
            end				
				
			elseif(not self.deleting) then
            -- we are not swapping, but also not deleting, so clear the flags
		      self.draggedIndex = 0;
		      self.swappingShortcuts = false;
			end
         
         -- clear the deleting flag no matter what
         self.deleting = false;
 	else
 	end
		end
      		
		self.quickslots[i].MouseDown = function(sender,args)
		   -- mouse down means we might be trying a drag, set some variables
 	if (not self.lockbar) then
			self.draggingShortcut = true;
			self.draggedIndex = i;
			
			-- for now, make sure we are not trying to swap until we know for sure
			self.swappingShortcuts = false;
 	else
 	end
		end

		self.quickslots[i].MouseLeave = function(sender,args)
		   -- if the mouse button is down, and we leave the quickslot, then we
		   -- are dragging the shortcut out
 		 if (not self.lockbar) then
			if(self.draggingShortcut) then
				self.draggedOut = true;
		 else
		 end
		end
	end

		self.quickslots[i].MouseUp = function(sender,args)
 		 if (not self.lockbar) then
			  if(args.Button == Turbine.UI.MouseButton.Left) then
				if(self.draggedOut) then
				   -- let the ShortcutChanged event know this is a delete event
				   -- then remove the shortcut
				   self.deleting = true;
					self:RemoveShortcut(i);
		 	  else
 			  end
		 end				
		end
			
			-- reset the tracking variables
			self.draggedOut=false;
			self.draggingShortcut = false;
		end
		
		self.quickslots[i].DragEnter = function(sender,args)
 		  if (not self.lockbar) then
         -- get the shortcut of the quickslot we just entered
		   self.ReplacedShortcut = self.quickslots[i]:GetShortcut();
         -- if the shortcut is a real shortcut, then we need to do a swap 
		     if(self.ReplacedShortcut:GetData() ~= "") then
            self.swappingShortcuts = true;
		    else
		      self.swappingShortcuts = false;
		    end
		  else
		  end
		end
		
		self.quickslots[i].DragDrop = function(sender,args)
		   --Turbine.Shell.WriteLine("I'm a quickslot drop event");
		end
	end
end

function pagedSlotsWindow:RemoveShortcut(i)
  if (not self.lockbar) then
      if (not self.settings.locked) then
	  local data = "";
	  local type = 0;
		
	  self.settings.shortcuts[i + (self.pageNumber - 1) * 15] = { Data=data,Type=type };
	
	  self:SaveSettings();
	  self:SetQuickslots();
      end
  else
  end
end


function pagedSlotsWindow:ChangeShortcut(i)
  if (not self.lockbar) then
  	local shortcut = self.quickslots[i]:GetShortcut();
	local data = shortcut:GetData();
	local type = shortcut:GetType(); 
	
	self.settings.shortcuts[i + (self.pageNumber - 1) * 15] = { Data=data,Type=type };
	
	self:SaveSettings();
	self:SetQuickslots();
  else
  end
end








-- function to load the settings from disk
function pagedSlotsWindow:LoadSettings()
	-- load the settings.  If a value is not available, set a default value
	self.loading = true;

	local settingStrings = Turbine.PluginData.Load( Turbine.DataScope.Character, "lockSlots2WindowSettings" );
	self.settings = { };

	if ( type( settingStrings ) ~= "table" ) then
		settingStrings = { };
	end
	
	-- check if the data exists, and create default values if not
	if ( not settingStrings.positionX or settingStrings.positionX == "nil" ) then
		settingStrings.positionX = tostring(Turbine.UI.Display.GetWidth() / 2 - self:GetWidth() - 226); 
	end

	if ( not settingStrings.positionY or settingStrings.positionY == "nil" ) then
		settingStrings.positionY = tostring(Turbine.UI.Display.GetHeight() *.66 - self:GetHeight() + 32);
	end

	if ( not settingStrings.orientation or settingStrings.orientation == "nil" ) then
		settingStrings.orientation = tostring(1);
	end

	if ( not settingStrings.shortcuts ) then
		settingStrings.shortcuts = { };		
		for i = 1, 120, 1 do
			local shortcutData = { Type="0",Data="" };
			table.insert(settingStrings.shortcuts,shortcutData);					
		end
	end
	
	-- convert from strings if necessary   
   if(type( settingStrings.positionX ) == "string" ) then
	   self.settings.positionX = tonumber(settingStrings.positionX);
	else
	   self.settings.positionX = settingStrings.positionX;
   end  
     
   if(type( settingStrings.positionY ) == "string" ) then
	   self.settings.positionY = tonumber(settingStrings.positionY);
	else
	   self.settings.positionY = settingStrings.positionY;
   end
     
   if(type( settingStrings.orientation ) == "string" ) then
	   self.settings.orientation = tonumber(settingStrings.orientation);
	else
	   self.settings.orientation = settingStrings.orientation;
   end
   
   -- check if the shortcut table needs to be changed to numbers
   local convertTableIndex = false;
   for i,v in pairs(settingStrings.shortcuts) do
      if(type(i) == "string") then
         convertTableIndex = true;
      end
   end
   
   -- convert the shortcut table index to numbers
   local tempTable = { };
   if(convertTableIndex) then       
      for i,v in pairs(settingStrings.shortcuts) do
         tempTable[tonumber(i)] = v;
      end    
   else
      tempTable = settingStrings.shortcuts;
   end   
	
	-- convert the table data to numbers
	self.settings.shortcuts = { };
	for i = 1 , 120 ,1 do
	   local shortcutData;
	   if(type( tempTable[i].Type) == "string" ) then
	      shortcutData = { Type= tonumber(tempTable[i].Type),Data=tempTable[i].Data };
	   else
	      shortcutData = { Type= tempTable[i].Type,Data=tempTable[i].Data };	   
	   end
	   
	   table.insert(self.settings.shortcuts,shortcutData);
	end

	self.loading = false;
end

function pagedSlotsWindow:SaveSettings()
	if (self.loading) then
		return;
	end
	
	settingStrings = { };
	
	-- convert the data to strings
   settingStrings.positionX = tostring(self.settings.positionX);
   settingStrings.positionY = tostring(self.settings.positionY);
   settingStrings.orientation = tostring(self.settings.orientation);
   
   settingStrings.shortcuts = { };

   -- loop through the shortcuts table and change all numbers to strings (index and type)
   for i,v in ipairs(self.settings.shortcuts) do
      typeNumber = v.Type;
      dataString = v.Data;
      settingStrings.shortcuts[tostring(i)] = {Type=tostring(typeNumber),Data=dataString};
   end

	-- save the settings
	Turbine.PluginData.Save( Turbine.DataScope.Character, "lockSlots2WindowSettings", settingStrings );
end

function pagedSlotsWindow:ChangeVisible(value)
	self.quickslots[1]:SetVisible(value);
end

-- function to clear all the quickslot on close to avoid a crash issue
-- (Turbine bug)
function pagedSlotsWindow:Closing(sender,args)
   for i = 1,15,1 do
      self.quickslots[i]:SetShortcut(nil);
   end
end